//= require <dSS/data/ModulatorStore>
//= require <dSS/data/DeviceStore>
//= require <dSS/data/ZoneStore>
//= require <dSS/data/GroupStore>
//= require <dSS/data/SceneStore>

Ext.namespace('dSS', 'dSS.data');

dSS.data.LoaderImpl = Ext.extend(Ext.util.Observable, {

    constructor: function() {
        this.modulatorStore = new dSS.data.ModulatorStore({
            url: '/json/apartment/getCircuits'
        });
        this.deviceStore = new dSS.data.DeviceStore();
        this.zoneStore = new dSS.data.ZoneStore();
        this.sceneStore = new dSS.data.SceneStore();
        this.groupStore = new dSS.data.GroupStore();

        this.modulatorStore.on('loadcomplete',
                               this.loadStructure.createDelegate(this));

        dSS.MessageBus.on('reloadapartment', this.load.createDelegate(this),
                           this);
    },

    load: function() {
        if (this.modulatorStore !== undefined) {
            this.modulatorStore.load();
        } else {
            dSS.MessageBus.fireEvent('structureloaded');
        }
    },

    getModulatorStore: function() {
        return this.modulatorStore;
    },

    getDeviceStore: function() {
        return this.deviceStore;
    },

    getZoneStore: function() {
        return this.zoneStore;
    },

    getSceneStore: function() {
        return this.sceneStore;
    },

    getGroupStore: function() {
        return this.groupStore;
    },

    loadStructure: function() {
        Ext.Ajax.request({
            url: '/json/apartment/getStructure',
            disableCaching: true,
            timeout: 300000,
            method: "GET",
            scope: this,
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    this.processStructure(jsonData.result);
                    this.loadClusterLocks();
                }
                catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not load apartment structure") + ": \"" + err + "\"");
                    dSS.MessageBus.fireEvent('structureloaded');
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not load") + ": \"" + request.url + "\"");
                dSS.MessageBus.fireEvent('structureloaded');
            }
        });
    },

    processStructure: function(structure) {
        var devices = [];
        var zones = [];
        var groups = [];
        var zoneStore = this.zoneStore;
        var modulatorStore = this.modulatorStore;
        var deviceStore = this.deviceStore;
        var groupStore = this.groupStore;

        Ext.each(structure.apartment.zones, function(zone) {
            if (zone.id === 0) {
                zone.name = zone.name ? zone.name: dSS._("All devices");
                Ext.each(zone.devices, function(device) {
                    device.firstSeen = Date.parseDate(device.firstSeen,
                                                      "Y-m-d H:i:s", true);
                    device.inactiveSince = Date.parseDate(
                                                device.inactiveSince,
                                                "Y-m-d H:i:s", true);
                    devices.push(device);
                });

                groups.push({
                    id: 0,
                    sortorder: 0,
                    color: -1,
                    name: dSS._("All devices")
                });

                for (var z = 0; z < zone.groups.length; z++) {
                    var grp = zone.groups[z];

                    if ((grp.id !== 0) &&
                        (grp.isValid === true) &&
                        (grp.devices.length > 0) &&
                        !dSS.util.isUserGroup(grp.id)) {
                        var name = grp.name ? grp.name: dSS._("Group") +
                                   ' #' + grp.id;
                        name = dSS._(dSS.config.groupNames[grp.id]);
                        var order = grp.id;

                        groups.push({
                            id: grp.id,
                            color: grp.color,
                            sortorder: order,
                            css: dSS.config.GroupColorIconCls[grp.color] +
                                "Small",
                            name: name
                        });
                    }
                }
            }

            var unnamedInZone = false;
            Ext.each(zone.devices, function(device) {
                if ((device.name === undefined) || (device.name.length === 0)) {
                    unnamedInZone = true;
                }
                var i;
                for (i = 0; i < devices.length; i++) {
                    if (devices[i].dSUID === device.dSUID) {
                        devices[i].zoneId = zone.id;
                        devices[i].zoneName = zone.name;
                    }
                }
            });

            zones.push({
                id: zone.id,
                unnamedDevices: unnamedInZone,
                alternativeSort: zone.id === 0 ? 1 : 0,
                name: zone.name ? zone.name: dSS._("Zone") + ' #' + zone.id,
                primary: ('firstZoneOnDSMeter' in zone) ? true: false
            });

            zoneStore.loadData({
                zones: zones
            });
        });

        Ext.each(structure.apartment.clusters, function(cluster) {
            var order = 200;
            if (cluster.isValid === true) {
                var name = cluster.name ? cluster.name: dSS._("Group") +
                           ' #' + cluster.id;
                if (cluster.isAutomatic === true) {
                    order = 400;
                }

                groups.push({
                    id: cluster.id,
                    color: cluster.color,
                    locked: cluster.configurationLock,
                    isAutomatic: cluster.isAutomatic,
                    protection: cluster.ProtectionClass,
                    direction: cluster.CardinalDirection,
                    isAutomatic: cluster.isAutomatic,
                    sortorder: order,
                    css: cluster.configurationLock ?
                                dSS.config.GroupColorLockCls[cluster.color] :
                                dSS.config.GroupColorIconCls[cluster.color] +
                                "UserGroup",
                    name: name
                });
            }
        });

        Ext.each(devices, function(device) {
            var index = modulatorStore.find('dsuid',
                        new RegExp("^" + device.meterDSUID + "$"));
            if (index > - 1) {
                var modulator = modulatorStore.getAt(index);
                device.meterName = modulator.get("name");
            }
        });

        groupStore.loadData({
            groups: groups
        });

        deviceStore.loadData({
            devices: devices
        });
        modulatorStore.each(
            function(rec) {
                rec.set('numDevices',
                        deviceStore.queryBy(function(record, id) {
                            if (record.get('meterDSUID') == rec.get('dsuid')) {
                                return true;
                            }
                            return false;
                        }).length);
            });
        modulatorStore.commitChanges();
        this.sceneStore.loadScenes();
    },
    processClusterLocks: function(json) {
        Ext.each(json.lockedDevices, function(device) {
            var store = dSS.data.Loader.getDeviceStore();
            var i = store.findExact('dsuid', device.dsuid);
            if (i < 0) {
                return;
            }

            var record = store.getAt(i);
            if (!record) {
                return;
            }

            record.set('clusterLock', true);
            record.set('lockedScenes', device.lockedScenes);
            record.commit();
        });

        Ext.each(json.lockedZones, function(zone) {
            var record = dSS.data.Loader.getZoneStore().getById(zone.zoneID);
            if (!record) {
                return;
            }

            record.set('clusterLocks', zone.deviceClasses);
            record.set('lockedApartmentScenes', zone.lockedApartmentScenes);
            record.commit();
        });
    },
    loadClusterLocks: function() {
        Ext.Ajax.request({
            url: '/json/apartment/getClusterLocks',
            disableCaching: true,
            timeout: 300000,
            method: "GET",
            scope: this,
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    this.processClusterLocks(jsonData.result);
                    dSS.MessageBus.fireEvent('structureloaded');
                }
                catch(err) {
                    dSS.MessageBus.fireEvent('structureloaded');
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not load cluster lock information") + ": \"" + err + "\"");
                }
            },
            failure: function(result, request) {
                dSS.MessageBus.fireEvent('structureloaded');
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not load cluster lock information"));
            }
        });
    }
});

